/*
**  wt -- a 3d game engine
**
**  Copyright (C) 1994 by Chris Laurel
**  email:  claurel@mr.net
**  snail mail:  Chris Laurel, 5700 W Lake St #208,  St. Louis Park, MN  55416
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include "wt.h"
#include "error.h"
#include "wtmem.h"


/* malloc or die . . . */
void *wtmalloc(size_t size)
{
     void *new;

     if ((new = malloc(size)) == NULL)
	  fatal_error("out of memory (wtmalloc %x bytes)", size);

     return new;
}


/* realloc or die . . . */
void *wtrealloc(void *v, size_t size)
{
     void *new;

     if ((new = realloc(v, size)) == NULL)
	  fatal_error("out of memory (wtrealloc %x bytes)", size);

     return new;
}


void wtfree(void *v)
{
     if (v == NULL)
	  fatal_error("wtfree(NULL)");
     else
	  free(v);
}
