/*
**  wt -- a 3d game engine
**
**  Copyright (C) 1994 by Chris Laurel
**  email:  claurel@mr.net
**  snail mail:  Chris Laurel, 5700 W Lake St #208,  St. Louis Park, MN  55416
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include <math.h>
#include "wt.h"
#include "error.h"
#include "fixed.h"
#include "wtmem.h"
#include "view.h"


View *new_view(fixed arc)
{
     View *v;

     if (arc >= FIXED_PI)
	  return NULL;
     v = wtmalloc(sizeof(View));
     v->arc = arc;
     v->eye_distance = FLOAT_TO_FIXED(1.0);
     v->view_plane_size =
	  fixmul(FLOAT_TO_FIXED(tan(FIXED_TO_FLOAT(arc) / 2.0)), 
		 v->eye_distance);

     return v;
}
